import 'dart:async';

import 'package:anyline/constants.dart';
import 'package:anyline/exceptions.dart';
import 'package:flutter/services.dart';
import 'package:permission_handler/permission_handler.dart';

/// Entrypoint for perfoming any scans using the Anyline OCR library.
class AnylinePlugin {
  
  static const MethodChannel _channel = const MethodChannel('anyline_plugin');

  AnylinePlugin();

  /// Returns the Anyline SDK version the plugin currently is powered by.
  static Future<String> get sdkVersion async {
    final String version =
        await _channel.invokeMethod(Constants.METHOD_GET_SDK_VERSION);
    return version;
  }
  
  /// Starts the Anyline SDK and invokes the scanning process with the given [configJson].
  ///
  /// Returns the result as a JSON string which can be parsed into an object of
  /// your choice. For more information about the possible forms of the JSON result
  /// and how to parse it, visit the [Anyline Documentation](https://documentation.anyline.com).
  /// 
  /// Uses the third-party-package `permission_handler` to request camera permissions.
  Future<String> startScanning(String configJson) async {
    if (await Permission.camera.isPermanentlyDenied) {
      openAppSettings();
    } else if (await Permission.camera.request().isGranted) {
      final Map<String, String> config = {
        Constants.EXTRA_CONFIG_JSON: configJson
      };
      try {
        final String result =
            await _channel.invokeMethod(Constants.METHOD_START_ANYLINE, config);
        return result;
      } on PlatformException catch (e) {
        throw AnylineException.parse(e);
      }
    } else {
      throw AnylineCameraPermissionException('Camera permission missing.');
    }
  }
}
